; Useful mathematics routines for 6809
; Alan Bleasby - December 2021
; Permission is given to freely use and distribute. Please give
; attribution.
; There is no warranty of any kind with this software

	section	"Maths"

	data

RESULT	rmb	$04		; Result of multiplications
MULPLC	rmb	$04		; Multiplicand storage
MULPLR	rmb	$02		; Multiplier storage
QUOT				; Quotient & divident are synonyms here
DVDEND	rmb	$04		; Dividend (and, afterwards, quotient)
DVSOR	rmb	$04		; Divisor
REMAIN	rmb	$04		; Remainder of division
MCOUNT	rmb	$01		; Counter byte for multiplications

	code


; M1616
; 16x16 multiply - 32 bit result
;
; INPUT
;   Multiplicand in MULPLC,+1. Multiplier in MULPLR,+1
; OUTPUT
;   Result in RESULT,+1,+2,+3
; Alters: None

M1616
	pshs	a,b

	ldd	#$0000
	std	RESULT,pcr
	std	RESULT+2,pcr

	ldb	#$10
1
	ror	MULPLR,pcr	; Test next bit of multiplier
	ror	MULPLR+1,pcr
	bcc	2F		; Don't add multiplicand

	lda	RESULT+3,pcr	; Add multiplicand to result
	adda	MULPLC+3,pcr
	sta	RESULT+3,pcr

	lda	RESULT+2,pcr
	adca	MULPLC+2,pcr
	sta	RESULT+2,pcr

	lda	RESULT+1,pcr
	adca	MULPLC+1,pcr
	sta	RESULT+1,pcr

	lda	RESULT,pcr
	adca	MULPLC,pcr
	sta	RESULT,pcr

2	asl	MULPLC+3,pcr	; Shift multiplicand left
	rol	MULPLC+2,pcr
	rol	MULPLC+1,pcr
	rol	MULPLC,pcr

	decb
	bne	1B

	puls	a,b,pc




; D1616
; 16x16 division
;
; INPUT
;   Dividend in DVDEND+1. Divisor in DVSOR,+1
; OUTPUT
;   Quotient in DVDEND+1. Remainder in REMAIN,+1
; Alters: None

D1616
	pshs	a,b
	ldd	#$0000
	std	REMAIN,pcr

	ldb	#$10
	stb	MCOUNT,pcr

1	asl	DVDEND+1,pcr	; xfer hi bit into remainder
	rol	DVDEND,pcr
	rol	REMAIN+1,pcr
	rol	REMAIN,pcr

	lda	REMAIN+1,pcr
	suba	DVSOR+1,pcr
	tfr	a,b
	lda	REMAIN,pcr
	sbca	DVSOR,pcr
	bcs	2F
	stb	REMAIN+1,pcr
	sta	REMAIN,pcr

	inc	DVDEND+1,pcr

2	dec	MCOUNT,pcr
	bne	1B

	puls	a,b,pc




; D2424
; 24x24 division
;
; INPUT
;   Dividend in DVDEND+1,+2. Divisor in DVSOR,+1,+2
; OUTPUT
;   Quotient in DVDEND+1,+2. Remainder in REMAIN,+1,+2
; Alters: None

D2424
	pshs	a,b

	ldd	#$0000
	std	REMAIN,pcr
	sta	REMAIN+2,pcr

	ldb	#$18
	stb	MCOUNT,pcr

1	asl	DVDEND+2,pcr
	rol	DVDEND+1,pcr
	rol	DVDEND,pcr
	rol	REMAIN+2,pcr
	rol	REMAIN+1,pcr
	rol	REMAIN,pcr

	lda	REMAIN+2,pcr
	suba	DVSOR+2,pcr
	lda	REMAIN+1,pcr
	sbca	DVSOR+1,pcr
	lda	REMAIN,pcr
	sbca	DVSOR,pcr

	bcs	2F
	
	lda	REMAIN+2,pcr
	suba	DVSOR+2,pcr
	sta	REMAIN+2,pcr

	lda	REMAIN+1,pcr
	sbca	DVSOR+1,pcr
	sta	REMAIN+1,pcr

	lda	REMAIN,pcr
	sbca	DVSOR,pcr
	sta	REMAIN,pcr

	inc	DVDEND+2,pcr

2	dec	MCOUNT,pcr
	bne	1B

	puls	a,b,pc




; D3232
; 32x32 division
;
; INPUT
;   Dividend in DVDEND+1,+2,+3. Divisor in DVSOR,+1,+2,+3
; OUTPUT
;   Quotient in DVDEND+1,+2,+3. Remainder in REMAIN,+1,+2,+3
; Alters: None

D3232
	pshs	a,b

	ldd	#$0000
	std	REMAIN,pcr
	std	REMAIN+2,pcr

	ldb	#$20
	stb	MCOUNT,pcr

1	asl	DVDEND+3,pcr
	rol	DVDEND+2,pcr
	rol	DVDEND+1,pcr
	rol	DVDEND,pcr
	rol	REMAIN+3,pcr
	rol	REMAIN+2,pcr
	rol	REMAIN+1,pcr
	rol	REMAIN,pcr

	lda	REMAIN+3,pcr
	suba	DVSOR+3,pcr
	lda	REMAIN+2,pcr
	sbca	DVSOR+2,pcr
	lda	REMAIN+1,pcr
	sbca	DVSOR+1,pcr
	lda	REMAIN,pcr
	sbca	DVSOR,pcr

	bcs	2F
	
	lda	REMAIN+3,pcr
	suba	DVSOR+3,pcr
	sta	REMAIN+3,pcr

	lda	REMAIN+2,pcr
	sbca	DVSOR+2,pcr
	sta	REMAIN+2,pcr

	lda	REMAIN+1,pcr
	sbca	DVSOR+1,pcr
	sta	REMAIN+1,pcr

	lda	REMAIN,pcr
	sbca	DVSOR,pcr
	sta	REMAIN,pcr

	inc	DVDEND+3,pcr

2	dec	MCOUNT,pcr
	bne	1B

	puls	a,b,pc
